package com.fzm.common.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.fzm.common.entity.Order;
import com.fzm.common.entity.dto.OrderDto;
import com.fzm.common.entity.vo.OrderVo;
import com.fzm.common.enums.OrderStatus;

import java.io.IOException;
import java.util.List;

/**
 * @author tangtuo
 * @date 2022/1/19 15:41
 */
public interface OrderService extends IService<Order> {

    /**
     * 根据支付场景和产品id查询订单
     *
     * @param payScene
     * @param productId
     * @param orderStatus
     * @return
     */
    Order getByPaySceneAndProductId(Integer payScene, Integer productId, Integer orderStatus);

    /**
     * 更新订单状态
     *
     * @param out_trade_no
     * @param orderStatus
     */
    void updateOrderStatus(Long out_trade_no, OrderStatus orderStatus);

    /**
     * 下单
     *
     * @param orderDto
     * @return
     */
    Order createOrder(OrderDto orderDto);

    /**
     * @param orderId
     * @param payType
     */
    Boolean updatePayType(Long orderId, Integer payType);

    /**
     * 取消订单
     *
     * @param orderStatus
     * @param orderId
     * @return
     */
    Boolean cancel(Long orderId, OrderStatus orderStatus);

    /**
     * 获取我的订单
     *
     * @param payScene
     * @param orderStatus
     * @return
     */
    List<OrderVo> listOrder(Integer payScene, Integer orderStatus);

    void processOrder(Long orderId, Integer productId, Integer payScene);

    /**
     * 更新订单状态
     *
     * @param orderId
     * @return
     */
    Boolean renewOrderStatus(Long orderId) throws IOException, InterruptedException;

    /**
     * 查询超时的订单
     *
     * @param hours
     * @return
     */
    List<Order> getTimeOutOrders(int hours);
}
